@@ -186,6 +186,15 @@ impl<T: NativeType> MutableBuffer<T> {
         self.len = 0
     }
 
+    /// Shortens the buffer.
+    /// If `len` is greater or equal to the buffers' current length, this has no effect.
+    #[inline]
+    pub fn truncate(&mut self, len: usize) {
+        if len < self.len {
+            self.len = len;
+        }
+    }
+
     /// Returns the data stored in this buffer as a slice.
     #[inline]
     pub fn as_slice(&self) -> &[T] {
@@ -363,6 +372,8 @@ impl<T: NativeType> MutableBuffer<T> {
     /// # Safety
     /// This method assumes that the iterator's size is correct and is undefined behavior
     /// to use it on an iterator that reports an incorrect length.
+    // This inline has been validated to offer 50% improvement in operations like `take`.
+    #[inline]
     pub unsafe fn extend_from_trusted_len_iter_unchecked<I: Iterator<Item = T>>(
         &mut self,
         iterator: I,
@@ -452,6 +463,8 @@ impl<T: NativeType> MutableBuffer<T> {
     /// # Safety
     /// This method assumes that the iterator's size is correct and is undefined behavior
     /// to use it on an iterator that reports an incorrect length.
+    // This inline has been validated to offer 50% improvement in operations like `take`.
+    #[inline]
     pub unsafe fn try_from_trusted_len_iter_unchecked<
         E,
         I: Iterator<Item = std::result::Result<T, E>>,
@@ -621,21 +634,21 @@ mod tests {
     fn default() {
         let b = MutableBuffer::<i32>::default();
         assert_eq!(b.len(), 0);
-        assert_eq!(b.is_empty(), true);
+        assert!(b.is_empty());
     }
 
     #[test]
     fn with_capacity() {
         let b = MutableBuffer::<i32>::with_capacity(6);
         assert!(b.capacity() >= 6);
-        assert_eq!(b.is_empty(), true);
+        assert!(b.is_empty());
     }
 
     #[test]
     fn from_len_zeroed() {
         let b = MutableBuffer::<i32>::from_len_zeroed(3);
         assert_eq!(b.len(), 3);
-        assert_eq!(b.is_empty(), false);
+        assert!(!b.is_empty());
         assert_eq!(b.as_slice(), &[0, 0, 0]);
     }
 
