@@ -90,11 +90,9 @@ fn filter_growable<'a>(growable: &mut impl Growable<'a>, chunks: &[(usize, usize
 
 macro_rules! dyn_build_filter {
     ($ty:ty, $array:expr, $filter_count:expr, $chunks:expr) => {{
-        let array = $array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<$ty>>()
-            .unwrap();
-        let mut growable = growable::GrowablePrimitive::<$ty>::new(&[array], false, $filter_count);
+        let array = $array.as_any().downcast_ref().unwrap();
+        let mut growable =
+            growable::GrowablePrimitive::<$ty>::new(vec![array], false, $filter_count);
         filter_growable(&mut growable, &$chunks);
         let array: PrimitiveArray<$ty> = growable.into();
         Box::new(array)
@@ -154,7 +152,7 @@ pub fn build_filter(filter: &BooleanArray) -> Result<Filter> {
         }
         DataType::Utf8 => {
             let array = array.as_any().downcast_ref::<Utf8Array<i32>>().unwrap();
-            let mut growable = growable::GrowableUtf8::<i32>::new(&[array], false, filter_count);
+            let mut growable = growable::GrowableUtf8::<i32>::new(vec![array], false, filter_count);
             filter_growable(&mut growable, &chunks);
             let array: Utf8Array<i32> = growable.into();
             Box::new(array)
@@ -254,14 +252,23 @@ pub fn filter(array: &dyn Array, filter: &BooleanArray) -> Result<Box<dyn Array>
 /// Therefore, it is considered undefined behavior to pass `filter` with null values.
 pub fn filter_record_batch(
     record_batch: &RecordBatch,
-    filter: &BooleanArray,
+    filter_values: &BooleanArray,
 ) -> Result<RecordBatch> {
-    let filter = build_filter(filter)?;
-    let filtered_arrays = record_batch
-        .columns()
-        .iter()
-        .map(|a| filter(a.as_ref()).into())
-        .collect();
+    let num_colums = record_batch.columns().len();
+
+    let filtered_arrays = match num_colums {
+        1 => {
+            vec![filter(record_batch.columns()[0].as_ref(), filter_values)?.into()]
+        }
+        _ => {
+            let filter = build_filter(filter_values)?;
+            record_batch
+                .columns()
+                .iter()
+                .map(|a| filter(a.as_ref()).into())
+                .collect()
+        }
+    };
     RecordBatch::try_new(record_batch.schema().clone(), filtered_arrays)
 }
 
@@ -316,9 +323,9 @@ mod tests {
         assert_eq!(67, d.len());
         assert_eq!(3, d.null_count());
         assert_eq!(1, d.value(0));
-        assert_eq!(true, d.is_null(1));
+        assert!(d.is_null(1));
         assert_eq!(64, d.value(63));
-        assert_eq!(true, d.is_null(64));
+        assert!(d.is_null(64));
         assert_eq!(67, d.value(65));
     }
 
@@ -344,7 +351,7 @@ mod tests {
         let c = filter(&a, &b).unwrap();
         let d = c.as_ref().as_any().downcast_ref::<Int32Array>().unwrap();
         assert_eq!(1, d.len());
-        assert_eq!(true, d.is_null(0));
+        assert!(d.is_null(0));
     }
 
     #[test]
@@ -359,8 +366,8 @@ mod tests {
             .unwrap();
         assert_eq!(2, d.len());
         assert_eq!("hello", d.value(0));
-        assert_eq!(false, d.is_null(0));
-        assert_eq!(true, d.is_null(1));
+        assert!(!d.is_null(0));
+        assert!(d.is_null(1));
     }
 
     #[test]
@@ -376,8 +383,8 @@ mod tests {
             .unwrap();
         assert_eq!(2, d.len());
         assert_eq!(b"hello", d.value(0));
-        assert_eq!(false, d.is_null(0));
-        assert_eq!(true, d.is_null(1));
+        assert!(!d.is_null(0));
+        assert!(d.is_null(1));
     }
 
     /*
