@@ -1,6 +1,8 @@
+use futures::{pin_mut, Stream, StreamExt};
 use parquet2::{
     encoding::{bitpacking, hybrid_rle, uleb128, Encoding},
-    read::{levels, FixedLenByteArrayPageDict, Page, PageHeader, StreamingIterator},
+    page::{DataPage, DataPageHeader, DataPageHeaderExt, FixedLenByteArrayPageDict},
+    read::{levels, StreamingIterator},
 };
 
 use super::{ColumnChunkMetaData, ColumnDescriptor};
@@ -136,7 +138,7 @@ pub fn iter_to_array<I, E>(
 where
     ArrowError: From<E>,
     E: Clone,
-    I: StreamingIterator<Item = std::result::Result<Page, E>>,
+    I: StreamingIterator<Item = std::result::Result<DataPage, E>>,
 {
     let capacity = metadata.num_values() as usize;
     let mut values = MutableBuffer::<u8>::with_capacity(capacity * size as usize);
@@ -158,8 +160,41 @@ where
     ))
 }
 
+pub async fn stream_to_array<I, E>(
+    pages: I,
+    size: i32,
+    metadata: &ColumnChunkMetaData,
+) -> Result<FixedSizeBinaryArray>
+where
+    ArrowError: From<E>,
+    E: Clone,
+    I: Stream<Item = std::result::Result<DataPage, E>>,
+{
+    let capacity = metadata.num_values() as usize;
+    let mut values = MutableBuffer::<u8>::with_capacity(capacity * size as usize);
+    let mut validity = MutableBitmap::with_capacity(capacity);
+
+    pin_mut!(pages); // needed for iteration
+
+    while let Some(page) = pages.next().await {
+        extend_from_page(
+            page.as_ref().map_err(|x| x.clone())?,
+            size,
+            metadata.descriptor(),
+            &mut values,
+            &mut validity,
+        )?
+    }
+
+    Ok(FixedSizeBinaryArray::from_data(
+        DataType::FixedSizeBinary(size),
+        values.into(),
+        validity.into(),
+    ))
+}
+
 pub(crate) fn extend_from_page(
-    page: &Page,
+    page: &DataPage,
     size: i32,
     descriptor: &ColumnDescriptor,
     values: &mut MutableBuffer<u8>,
@@ -169,8 +204,8 @@ pub(crate) fn extend_from_page(
     assert!(descriptor.max_def_level() <= 1);
     let is_optional = descriptor.max_def_level() == 1;
     match page.header() {
-        PageHeader::V1(header) => {
-            assert_eq!(header.definition_level_encoding, Encoding::Rle);
+        DataPageHeader::V1(header) => {
+            assert_eq!(header.definition_level_encoding(), Encoding::Rle);
 
             let (_, validity_buffer, values_buffer) =
                 levels::split_buffer_v1(page.buffer(), false, is_optional);
@@ -207,7 +242,7 @@ pub(crate) fn extend_from_page(
                 }
             }
         }
-        PageHeader::V2(header) => {
+        DataPageHeader::V2(header) => {
             let def_level_buffer_length = header.definition_levels_byte_length as usize;
 
             match (page.encoding(), page.dictionary_page(), is_optional) {
