@@ -1,4 +1,30 @@
-use std::io::{Read, Seek};
+use std::{
+    io::{Read, Seek},
+    sync::Arc,
+};
+
+use futures::{AsyncRead, AsyncSeek, Stream};
+pub use parquet2::{
+    error::ParquetError,
+    metadata::{ColumnChunkMetaData, ColumnDescriptor, RowGroupMetaData},
+    page::{CompressedDataPage, DataPage, DataPageHeader},
+    read::{
+        decompress, get_page_iterator as _get_page_iterator, get_page_stream as _get_page_stream,
+        read_metadata as _read_metadata, read_metadata_async as _read_metadata_async,
+        streaming_iterator, Decompressor, PageFilter, PageIterator, StreamingIterator,
+    },
+    schema::types::{
+        LogicalType, ParquetType, PhysicalType, PrimitiveConvertedType,
+        TimeUnit as ParquetTimeUnit, TimestampType,
+    },
+    types::int96_to_i64_ns,
+};
+
+use crate::{
+    array::Array,
+    datatypes::{DataType, IntervalUnit, TimeUnit},
+    error::{ArrowError, Result},
+};
 
 mod binary;
 mod boolean;
@@ -10,231 +36,220 @@ pub mod schema;
 pub mod statistics;
 mod utils;
 
-use crate::{
-    array::Array,
-    datatypes::{DataType, TimeUnit},
-    error::{ArrowError, Result},
-};
-
 pub use record_batch::RecordReader;
 pub use schema::{get_schema, is_type_nullable, FileMetaData};
 
-pub use parquet2::{
-    error::ParquetError,
-    metadata::{ColumnChunkMetaData, ColumnDescriptor, RowGroupMetaData},
-    read::{
-        decompress, streaming_iterator, CompressedPage, Decompressor, Page, PageHeader,
-        StreamingIterator,
-    },
-    schema::{
-        types::{LogicalType, ParquetType, PhysicalType, PrimitiveConvertedType},
-        TimeUnit as ParquetTimeUnit, TimestampType,
-    },
-};
-use parquet2::{
-    read::{
-        get_page_iterator as _get_page_iterator, read_metadata as _read_metadata, PageIterator,
-    },
-    types::int96_to_i64_ns,
-};
-
 /// Creates a new iterator of compressed pages.
 pub fn get_page_iterator<'b, RR: Read + Seek>(
     metadata: &FileMetaData,
     row_group: usize,
     column: usize,
     reader: &'b mut RR,
+    pages_filter: Option<PageFilter>,
     buffer: Vec<u8>,
 ) -> Result<PageIterator<'b, RR>> {
     Ok(_get_page_iterator(
-        metadata, row_group, column, reader, buffer,
+        metadata,
+        row_group,
+        column,
+        reader,
+        pages_filter,
+        buffer,
     )?)
 }
 
-/// Reads parquets' metadata.
+/// Creates a new iterator of compressed pages.
+pub async fn get_page_stream<'a, RR: AsyncRead + Unpin + Send + AsyncSeek>(
+    metadata: &'a FileMetaData,
+    row_group: usize,
+    column: usize,
+    reader: &'a mut RR,
+    pages_filter: Option<PageFilter>,
+    buffer: Vec<u8>,
+) -> Result<impl Stream<Item = std::result::Result<CompressedDataPage, ParquetError>> + 'a> {
+    let pages_filter = pages_filter.unwrap_or_else(|| Arc::new(|_, _| true));
+    Ok(_get_page_stream(metadata, row_group, column, reader, buffer, pages_filter).await?)
+}
+
+/// Reads parquets' metadata syncronously.
 pub fn read_metadata<R: Read + Seek>(reader: &mut R) -> Result<FileMetaData> {
     Ok(_read_metadata(reader)?)
 }
 
-fn page_iter_i64<I: StreamingIterator<Item = std::result::Result<Page, ParquetError>>>(
-    iter: I,
-    metadata: &ColumnChunkMetaData,
-    data_type: DataType,
-) -> Result<Box<dyn Array>> {
-    if metadata.descriptor().max_rep_level() > 0 {
-        let inner = match data_type {
-            DataType::List(ref inner) => inner.data_type(),
-            DataType::LargeList(ref inner) => inner.data_type(),
-            _ => unimplemented!(),
-        };
-
-        match inner {
-            DataType::UInt64 => {
-                primitive::iter_to_array_nested(iter, metadata, data_type, |x: i64| x as u64)
-            }
-            _ => primitive::iter_to_array_nested(iter, metadata, data_type, |x: i64| x as i64),
-        }
-    } else {
-        match data_type {
-            DataType::UInt64 => {
-                primitive::iter_to_array(iter, metadata, data_type, |x: i64| x as u64)
-            }
-            _ => primitive::iter_to_array(iter, metadata, data_type, |x: i64| x as i64),
-        }
-    }
+/// Reads parquets' metadata asynchronously.
+pub async fn read_metadata_async<R: AsyncRead + AsyncSeek + Send + Unpin>(
+    reader: &mut R,
+) -> Result<FileMetaData> {
+    Ok(_read_metadata_async(reader).await?)
 }
 
-fn page_iter_i32<I: StreamingIterator<Item = std::result::Result<Page, ParquetError>>>(
-    iter: I,
+pub fn page_iter_to_array<
+    I: StreamingIterator<Item = std::result::Result<DataPage, ParquetError>>,
+>(
+    iter: &mut I,
     metadata: &ColumnChunkMetaData,
     data_type: DataType,
 ) -> Result<Box<dyn Array>> {
     use DataType::*;
+    match data_type {
+        // INT32
+        UInt8 => primitive::iter_to_array(iter, metadata, data_type, |x: i32| x as u8),
+        UInt16 => primitive::iter_to_array(iter, metadata, data_type, |x: i32| x as u16),
+        UInt32 => primitive::iter_to_array(iter, metadata, data_type, |x: i32| x as u32),
+        Int8 => primitive::iter_to_array(iter, metadata, data_type, |x: i32| x as i8),
+        Int16 => primitive::iter_to_array(iter, metadata, data_type, |x: i32| x as i16),
+        Int32 | Date32 | Time32(_) | Interval(IntervalUnit::YearMonth) => {
+            primitive::iter_to_array(iter, metadata, data_type, |x: i32| x as i32)
+        }
 
-    if metadata.descriptor().max_rep_level() > 0 {
-        let inner = match data_type {
-            List(ref inner) => inner.data_type(),
-            LargeList(ref inner) => inner.data_type(),
-            _ => unimplemented!(),
-        };
+        Timestamp(TimeUnit::Nanosecond, None) => match metadata.descriptor().type_() {
+            ParquetType::PrimitiveType { physical_type, .. } => match physical_type {
+                PhysicalType::Int96 => primitive::iter_to_array(
+                    iter,
+                    metadata,
+                    DataType::Timestamp(TimeUnit::Nanosecond, None),
+                    int96_to_i64_ns,
+                ),
+                _ => primitive::iter_to_array(iter, metadata, data_type, |x: i64| x),
+            },
+            _ => unreachable!(),
+        },
 
-        match inner {
+        // INT64
+        Int64 | Date64 | Time64(_) | Duration(_) | Timestamp(_, _) => {
+            primitive::iter_to_array(iter, metadata, data_type, |x: i64| x)
+        }
+        UInt64 => primitive::iter_to_array(iter, metadata, data_type, |x: i64| x as u64),
+
+        Float32 => primitive::iter_to_array(iter, metadata, data_type, |x: f32| x),
+        Float64 => primitive::iter_to_array(iter, metadata, data_type, |x: f64| x),
+
+        Boolean => Ok(Box::new(boolean::iter_to_array(iter, metadata)?)),
+
+        Binary | Utf8 => binary::iter_to_array::<i32, _, _>(iter, metadata, &data_type),
+        LargeBinary | LargeUtf8 => binary::iter_to_array::<i64, _, _>(iter, metadata, &data_type),
+        FixedSizeBinary(size) => Ok(Box::new(fixed_size_binary::iter_to_array(
+            iter, size, metadata,
+        )?)),
+
+        List(ref inner) => match inner.data_type() {
             UInt8 => primitive::iter_to_array_nested(iter, metadata, data_type, |x: i32| x as u8),
             UInt16 => primitive::iter_to_array_nested(iter, metadata, data_type, |x: i32| x as u16),
             UInt32 => primitive::iter_to_array_nested(iter, metadata, data_type, |x: i32| x as u32),
             Int8 => primitive::iter_to_array_nested(iter, metadata, data_type, |x: i32| x as i8),
             Int16 => primitive::iter_to_array_nested(iter, metadata, data_type, |x: i32| x as i16),
-            _ => primitive::iter_to_array_nested(iter, metadata, data_type, |x: i32| x),
-        }
-    } else {
-        match data_type {
-            UInt8 => primitive::iter_to_array(iter, metadata, data_type, |x: i32| x as u8),
-            UInt16 => primitive::iter_to_array(iter, metadata, data_type, |x: i32| x as u16),
-            UInt32 => primitive::iter_to_array(iter, metadata, data_type, |x: i32| x as u32),
-            Int8 => primitive::iter_to_array(iter, metadata, data_type, |x: i32| x as i8),
-            Int16 => primitive::iter_to_array(iter, metadata, data_type, |x: i32| x as i16),
-            _ => primitive::iter_to_array(iter, metadata, data_type, |x: i32| x),
-        }
-    }
-}
+            Int32 => primitive::iter_to_array_nested(iter, metadata, data_type, |x: i32| x as i32),
 
-fn page_iter_byte_array<I: StreamingIterator<Item = std::result::Result<Page, ParquetError>>>(
-    iter: I,
-    metadata: &ColumnChunkMetaData,
-    data_type: DataType,
-) -> Result<Box<dyn Array>> {
-    use DataType::*;
+            Timestamp(TimeUnit::Nanosecond, None) => match metadata.descriptor().type_() {
+                ParquetType::PrimitiveType { physical_type, .. } => match physical_type {
+                    PhysicalType::Int96 => primitive::iter_to_array_nested(
+                        iter,
+                        metadata,
+                        DataType::Timestamp(TimeUnit::Nanosecond, None),
+                        int96_to_i64_ns,
+                    ),
+                    _ => primitive::iter_to_array(iter, metadata, data_type, |x: i64| x),
+                },
+                _ => unreachable!(),
+            },
 
-    if metadata.descriptor().max_rep_level() > 0 {
-        let inner = match data_type {
-            List(ref inner) => inner.data_type(),
-            LargeList(ref inner) => inner.data_type(),
-            _ => unimplemented!(),
-        };
+            // INT64
+            Int64 | Date64 | Time64(_) | Duration(_) | Timestamp(_, _) => {
+                primitive::iter_to_array_nested(iter, metadata, data_type, |x: i64| x)
+            }
+            UInt64 => primitive::iter_to_array_nested(iter, metadata, data_type, |x: i64| x as u64),
+
+            Float32 => primitive::iter_to_array_nested(iter, metadata, data_type, |x: f32| x),
+            Float64 => primitive::iter_to_array_nested(iter, metadata, data_type, |x: f64| x),
+
+            Boolean => boolean::iter_to_array_nested(iter, metadata, data_type),
 
-        match inner {
             Binary | Utf8 => binary::iter_to_array_nested::<i32, _, _>(iter, metadata, data_type),
             LargeBinary | LargeUtf8 => {
                 binary::iter_to_array_nested::<i64, _, _>(iter, metadata, data_type)
             }
             other => Err(ArrowError::NotYetImplemented(format!(
-                "Can't read {:?} from parquet",
+                "The conversion of {:?} to arrow still not implemented",
                 other
             ))),
-        }
-    } else {
-        match data_type {
-            Binary | Utf8 => binary::iter_to_array::<i32, _, _>(iter, metadata, &data_type),
-            LargeBinary | LargeUtf8 => {
-                binary::iter_to_array::<i64, _, _>(iter, metadata, &data_type)
-            }
-            other => Err(ArrowError::NotYetImplemented(format!(
-                "Can't read {:?} from parquet",
-                other
-            ))),
-        }
+        },
+
+        Dictionary(ref key, ref values) => match key.as_ref() {
+            Int32 => match values.as_ref() {
+                Int32 => primitive::iter_to_dict_array::<i32, _, _, _, _, _>(
+                    iter,
+                    metadata,
+                    data_type,
+                    |x: i32| x as i32,
+                ),
+                _ => todo!(),
+            },
+            _ => todo!(),
+        },
+
+        other => Err(ArrowError::NotYetImplemented(format!(
+            "The conversion of {:?} to arrow still not implemented",
+            other
+        ))),
     }
 }
 
-fn page_iter_fixed_len_byte_array<
-    I: StreamingIterator<Item = std::result::Result<Page, ParquetError>>,
->(
-    iter: I,
+// Converts an async stream of compressed data pages into an [`Array`].
+pub async fn page_stream_to_array<I: Stream<Item = std::result::Result<DataPage, ParquetError>>>(
+    pages: I,
     metadata: &ColumnChunkMetaData,
     data_type: DataType,
 ) -> Result<Box<dyn Array>> {
     use DataType::*;
-    Ok(match data_type {
-        FixedSizeBinary(size) => Box::new(fixed_size_binary::iter_to_array(iter, size, metadata)?),
-        other => {
-            return Err(ArrowError::NotYetImplemented(format!(
-                "Can't read {:?} from parquet",
-                other
-            )))
+    match data_type {
+        // INT32
+        UInt8 => primitive::stream_to_array(pages, metadata, data_type, |x: i32| x as u8).await,
+        UInt16 => primitive::stream_to_array(pages, metadata, data_type, |x: i32| x as u16).await,
+        UInt32 => primitive::stream_to_array(pages, metadata, data_type, |x: i32| x as u32).await,
+        Int8 => primitive::stream_to_array(pages, metadata, data_type, |x: i32| x as i8).await,
+        Int16 => primitive::stream_to_array(pages, metadata, data_type, |x: i32| x as i16).await,
+        Int32 | Date32 | Time32(_) | Interval(IntervalUnit::YearMonth) => {
+            primitive::stream_to_array(pages, metadata, data_type, |x: i32| x as i32).await
         }
-    })
-}
 
-pub fn page_iter_to_array<I: StreamingIterator<Item = std::result::Result<Page, ParquetError>>>(
-    iter: &mut I,
-    metadata: &ColumnChunkMetaData,
-    data_type: DataType,
-) -> Result<Box<dyn Array>> {
-    match metadata.descriptor().base_type() {
-        ParquetType::PrimitiveType {
-            physical_type,
-            converted_type,
-            logical_type,
-            ..
-        } => match (physical_type, converted_type, logical_type) {
-            (PhysicalType::Int32, _, _) => page_iter_i32(iter, metadata, data_type),
-            (PhysicalType::Int64, _, _) => page_iter_i64(iter, metadata, data_type),
-            (PhysicalType::Float, None, None) => {
-                primitive::iter_to_array(iter, metadata, DataType::Float32, |x: f32| x)
-            }
-            (PhysicalType::Double, None, None) => {
-                primitive::iter_to_array(iter, metadata, DataType::Float64, |x: f64| x)
-            }
-            (PhysicalType::Boolean, None, None) => {
-                Ok(Box::new(boolean::iter_to_array(iter, metadata)?))
-            }
-            (PhysicalType::ByteArray, _, _) => page_iter_byte_array(iter, metadata, data_type),
-            (PhysicalType::FixedLenByteArray(_), _, _) => {
-                page_iter_fixed_len_byte_array(iter, metadata, data_type)
-            }
-            (PhysicalType::Int96, _, _) => primitive::iter_to_array(
-                iter,
-                metadata,
-                DataType::Timestamp(TimeUnit::Nanosecond, None),
-                int96_to_i64_ns,
-            ),
-            (p, c, l) => Err(ArrowError::NotYetImplemented(format!(
-                "The conversion of ({:?}, {:?}, {:?}) to arrow still not implemented",
-                p, c, l
-            ))),
+        Timestamp(TimeUnit::Nanosecond, None) => match metadata.descriptor().type_() {
+            ParquetType::PrimitiveType { physical_type, .. } => match physical_type {
+                PhysicalType::Int96 => {
+                    primitive::stream_to_array(
+                        pages,
+                        metadata,
+                        DataType::Timestamp(TimeUnit::Nanosecond, None),
+                        int96_to_i64_ns,
+                    )
+                    .await
+                }
+                _ => primitive::stream_to_array(pages, metadata, data_type, |x: i64| x).await,
+            },
+            _ => unreachable!(),
         },
-        _ => {
-            // get all primitives in the type_ and their max rep levels
-            match metadata.descriptor().type_() {
-                ParquetType::PrimitiveType {
-                    physical_type,
-                    converted_type,
-                    logical_type,
-                    ..
-                } => match (physical_type, converted_type, logical_type) {
-                    (PhysicalType::Boolean, None, None) => {
-                        boolean::iter_to_array_nested(iter, metadata, data_type)
-                    }
-                    (PhysicalType::Int64, _, _) => page_iter_i64(iter, metadata, data_type),
-                    (PhysicalType::Int32, _, _) => page_iter_i32(iter, metadata, data_type),
-                    (PhysicalType::ByteArray, _, _) => {
-                        page_iter_byte_array(iter, metadata, data_type)
-                    }
-                    _ => todo!(),
-                },
-                _ => unreachable!(),
-            }
+
+        // INT64
+        Int64 | Date64 | Time64(_) | Duration(_) | Timestamp(_, _) => {
+            primitive::stream_to_array(pages, metadata, data_type, |x: i64| x).await
         }
+        UInt64 => primitive::stream_to_array(pages, metadata, data_type, |x: i64| x as u64).await,
+
+        Float32 => primitive::stream_to_array(pages, metadata, data_type, |x: f32| x).await,
+        Float64 => primitive::stream_to_array(pages, metadata, data_type, |x: f64| x).await,
+
+        Boolean => Ok(Box::new(boolean::stream_to_array(pages, metadata).await?)),
+
+        Binary | Utf8 => binary::stream_to_array::<i32, _, _>(pages, metadata, &data_type).await,
+        LargeBinary | LargeUtf8 => {
+            binary::stream_to_array::<i64, _, _>(pages, metadata, &data_type).await
+        }
+        FixedSizeBinary(size) => Ok(Box::new(
+            fixed_size_binary::stream_to_array(pages, size, metadata).await?,
+        )),
+        other => Err(ArrowError::NotYetImplemented(format!(
+            "Async conversion of {:?}",
+            other
+        ))),
     }
 }
 
@@ -285,7 +300,7 @@ mod tests {
         };
 
         assert_eq!(expected.as_ref(), array.as_ref());
-        assert_eq!(expected_statistics.as_ref(), statistics.unwrap().as_ref());
+        assert_eq!(expected_statistics, statistics);
 
         Ok(())
     }
@@ -469,7 +484,7 @@ mod tests_integration {
         let path = "testing/parquet-testing/data/alltypes_plain.parquet";
         let reader = std::fs::File::open(path)?;
 
-        let reader = RecordReader::try_new(reader, None, None, Arc::new(|_, _| true))?;
+        let reader = RecordReader::try_new(reader, None, None, Arc::new(|_, _| true), None)?;
 
         let batches = reader.collect::<Result<Vec<_>>>()?;
         assert_eq!(batches.len(), 1);
