@@ -1,6 +1,7 @@
 use parquet2::{
     encoding::{bitpacking, hybrid_rle, uleb128, Encoding},
-    read::{levels, Page, PageHeader, PrimitivePageDict},
+    page::{DataPage, DataPageHeader, DataPageHeaderExt, PrimitivePageDict},
+    read::levels,
     types::NativeType,
 };
 
@@ -142,7 +143,7 @@ fn read_required<T, A, F>(
 }
 
 pub fn extend_from_page<T, A, F>(
-    page: &Page,
+    page: &DataPage,
     descriptor: &ColumnDescriptor,
     values: &mut MutableBuffer<A>,
     validity: &mut MutableBitmap,
@@ -158,22 +159,24 @@ where
     assert_eq!(descriptor.max_rep_level(), 0);
     let is_optional = descriptor.max_def_level() == 1;
     match page.header() {
-        PageHeader::V1(header) => {
-            assert_eq!(header.definition_level_encoding, Encoding::Rle);
+        DataPageHeader::V1(header) => {
+            assert_eq!(header.definition_level_encoding(), Encoding::Rle);
 
             let (_, validity_buffer, values_buffer) =
                 levels::split_buffer_v1(page.buffer(), false, is_optional);
 
             match (&page.encoding(), page.dictionary_page(), is_optional) {
-                (Encoding::PlainDictionary, Some(dict), true) => read_dict_buffer_optional(
-                    validity_buffer,
-                    values_buffer,
-                    additional,
-                    dict.as_any().downcast_ref().unwrap(),
-                    values,
-                    validity,
-                    op,
-                ),
+                (Encoding::PlainDictionary | Encoding::RleDictionary, Some(dict), true) => {
+                    read_dict_buffer_optional(
+                        validity_buffer,
+                        values_buffer,
+                        additional,
+                        dict.as_any().downcast_ref().unwrap(),
+                        values,
+                        validity,
+                        op,
+                    )
+                }
                 (Encoding::Plain, None, true) => read_nullable(
                     validity_buffer,
                     values_buffer,
@@ -196,21 +199,23 @@ where
                 }
             }
         }
-        PageHeader::V2(header) => {
+        DataPageHeader::V2(header) => {
             let def_level_buffer_length = header.definition_levels_byte_length as usize;
 
             let (_, validity_buffer, values_buffer) =
                 levels::split_buffer_v2(page.buffer(), 0, def_level_buffer_length);
             match (&page.encoding(), page.dictionary_page(), is_optional) {
-                (Encoding::PlainDictionary, Some(dict), true) => read_dict_buffer_optional(
-                    validity_buffer,
-                    values_buffer,
-                    additional,
-                    dict.as_any().downcast_ref().unwrap(),
-                    values,
-                    validity,
-                    op,
-                ),
+                (Encoding::PlainDictionary | Encoding::RleDictionary, Some(dict), true) => {
+                    read_dict_buffer_optional(
+                        validity_buffer,
+                        values_buffer,
+                        additional,
+                        dict.as_any().downcast_ref().unwrap(),
+                        values,
+                        validity,
+                        op,
+                    )
+                }
                 (Encoding::Plain, None, true) => read_nullable(
                     validity_buffer,
                     values_buffer,
